-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '2.4.2'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '2.5.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION




    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    -- REMOVEMOS LOS REGISTROS INNECESARIOS POR LA REFORMULACION DE AGENDA
    DELETE FROM [dbo].[CategoryPermissions] WHERE [Category] = 'CATEG_GROUP_SCHEDULE' AND [Position] = 7
    DELETE FROM [dbo].[RolePermissionsGroup] WHERE [Category] = 'CATEG_GROUP_SCHEDULE'
    DELETE FROM [dbo].[RoutePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_VIEW'
    DELETE FROM [dbo].[RoutePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_EDIT'
    DELETE FROM [dbo].[RoutePermission] WHERE [GroupId] = 'PERM_GROUP_STUDY_DELETE' AND [Route] = 'api/schedule/delete_schedule'
    DELETE FROM [dbo].[RolePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_ADD'
    DELETE FROM [dbo].[RolePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_EDIT'
    DELETE FROM [dbo].[RolePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_DELETE'
    DELETE FROM [dbo].[RolePermission] WHERE [GroupId] = 'PERM_GROUP_SCHEDULE_VIEW'

    -- REMOVEMOS LOS ACTUALES PERMISOS DE EXCEPCIONES
    DELETE FROM [RoutePermission] WHERE [Route] LIKE '%exception%'

    -- ACTUALIZAMOS LOS DEMAS ROLEPERMISSIONS PARA REDUCIR UNA POSICION
    UPDATE [dbo].[CategoryPermissions] SET [Position] = [Position] - 1 WHERE [Position] > 7
    
    -- INSERTAMOS PERMISOS QUE PODRIAN FALTAR EN LAS PLATAFORMAS INSTALADAS POR PRIMERA VEZ CON EL BUILD 0.16.0, EN CASO YA EXISTAN NO SE INSERTAN
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
    FOR 
    -- SELECCIONAMOS LOS VCC EXISTENTES
    SELECT [VCC]
        FROM [dbo].[VCC] (NOLOCK) 

    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
    DECLARE @CURRENT_VCC_TO_UPDATE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR

    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    WHILE @@FETCH_STATUS = 0
    BEGIN 
        -- EJECUTAMOS EL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE`
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_TEMPLATE', N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_TEMPLATE', N'PERM_GROUP_SCHEDULED_TEMPLATE_EDIT','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_TEMPLATE', N'PERM_GROUP_SCHEDULED_TEMPLATE_DELETE','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_TEMPLATE', N'PERM_GROUP_SCHEDULED_TEMPLATE_VIEW','SPEECH', GETUTCDATE())

        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD', N'speechanalytic/analytic/template/add-schedule', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD', N'api/template/save_template', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD', N'api/template/get_templates/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD', N'api/batch/check_batch_name', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_TEMPLATE_EDIT', N'api/schedule/save_schedule_status', N'api', GETUTCDATE())

        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_TEMPLATE_VIEW', N'api/schedule/get_template_schedules', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_TEMPLATE_VIEW', N'api/schedule/get_schedule_executions', N'api', GETUTCDATE())

        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_TEMPLATE_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_TEMPLATE_EDIT', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_TEMPLATE_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_TEMPLATE_VIEW', GETUTCDATE())

        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_PROCESS', N'PERM_GROUP_SCHEDULED_PROCESS_ADD','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_PROCESS', N'PERM_GROUP_SCHEDULED_PROCESS_EDIT','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_PROCESS', N'PERM_GROUP_SCHEDULED_PROCESS_DELETE','SPEECH', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_PROCESS', N'PERM_GROUP_SCHEDULED_PROCESS_VIEW','SPEECH', GETUTCDATE())

        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_VIEW', N'api/schedule/get_transcription_schedules', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_PROCESS_VIEW', N'api/schedule/get_schedule_executions', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SCHEDULED_PROCESS_EDIT', N'api/schedule/save_schedule_status', N'api', GETUTCDATE())
        
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_PROCESS_ADD', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_PROCESS_EDIT', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_PROCESS_DELETE', GETUTCDATE())
        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SCHEDULED_PROCESS_VIEW', GETUTCDATE())

        INSERT [dbo].[RolePermission] ([VCC], [RoleId], [GroupId], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'administrator', N'PERM_GROUP_SPEECH_DASHBOARD', GETUTCDATE())
        INSERT [dbo].[RolePermissionsGroup] ([VCC], [Category], [GroupId], [Type], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, N'CATEG_GROUP_HOME', N'PERM_GROUP_SPEECH_DASHBOARD', 'SPEECH', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/schedule/get_template_schedules', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/idatha_auth/get_idatha_auth_by_realm', N'api', GETUTCDATE())  
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/schedule/get_scheduled_template_results_summary', N'api', GETUTCDATE())

        -- REMOVEMOS LOS ACTUALES PERMISOS DE EXCEPCIONES
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'speechanalytic/home/exceptions', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/get_exceptions/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'speechanalytic/home/exceptions/add', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/create_exception', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'speechanalytic/home/exceptions/edit/', N'client', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/get_exception/', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/save_exception', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/delete_exception', N'api', GETUTCDATE())
        INSERT [dbo].[RoutePermission] ([VCC], [ID], [GroupId], [Route], [Level], [TimeStamp]) VALUES (@CURRENT_VCC_TO_UPDATE, NEWID(), N'PERM_GROUP_SPEECH_DASHBOARD', N'api/exception/delete_exceptions', N'api', GETUTCDATE())


        -- FIN DEL SCRIPT DE INSERT/UPDATE/DELETE PARA CADA VCC EN LA VARIABLE @CURRENT_VCC_TO_UPDATE

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_VCC_TO_UPDATE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    -- ACTUALIZAMOS EL VALOR DEL NUEVO CAMPO EN LA TABLA DE EJECUCIONES
    UPDATE [dbo].[BatchScheduleExecution] 
        SET [ProcessId] = [BatchId]
        WHERE [BatchId] IS NOT NULL
    
    

    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


    

    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, 
            [IsRegistered] = 0,
            [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH

